@@ -1,14 +1,15 @@
 package dnsforward
 
 import (
+	"context"
 	"fmt"
+	"log/slog"
 	"slices"
 	"time"
 
 	"github.com/AdguardTeam/AdGuardHome/internal/aghnet"
 	"github.com/AdguardTeam/dnsproxy/proxy"
 	"github.com/AdguardTeam/dnsproxy/upstream"
-	"github.com/AdguardTeam/golibs/log"
 	"github.com/AdguardTeam/golibs/netutil"
 	"github.com/AdguardTeam/golibs/stringutil"
 )
@@ -55,8 +56,10 @@ func newBootstrap(
 // newUpstreamConfig returns the upstream configuration based on upstreams.  If
 // upstreams slice specifies no default upstreams, defaultUpstreams are used to
 // create upstreams with no domain specifications.  opts are used when creating
-// upstream configuration.
+// upstream configuration.  l must not be nil.
 func newUpstreamConfig(
+	ctx context.Context,
+	l *slog.Logger,
 	upstreams []string,
 	defaultUpstreams []string,
 	opts *upstream.Options,
@@ -67,7 +70,12 @@ func newUpstreamConfig(
 	}
 
 	if len(uc.Upstreams) == 0 && len(defaultUpstreams) > 0 {
-		log.Info("dnsforward: warning: no default upstreams specified, using %v", defaultUpstreams)
+		l.WarnContext(
+			ctx,
+			"no default upstreams specified",
+			"default_upstreams",
+			defaultUpstreams,
+		)
 
 		var defaultUpstreamConfig *proxy.UpstreamConfig
 		defaultUpstreamConfig, err = proxy.ParseUpstreamsConfig(defaultUpstreams, opts)
@@ -84,8 +92,11 @@ func newUpstreamConfig(
 // newPrivateConfig creates an upstream configuration for resolving PTR records
 // for local addresses.  The configuration is built either from the provided
 // addresses or from the system resolvers.  unwanted filters the resulting
-// upstream configuration.
+// upstream configuration.  l, unwanted, sysResolvers, privateNets and opts must
+// not be nil.
 func newPrivateConfig(
+	ctx context.Context,
+	l *slog.Logger,
 	addrs []string,
 	unwanted addrPortSet,
 	sysResolvers SystemResolvers,
@@ -103,7 +114,7 @@ func newPrivateConfig(
 		}
 	}
 
-	log.Debug("dnsforward: private-use upstreams: %v", addrs)
+	l.DebugContext(ctx, "private-use upstreams", "addrs", addrs)
 
 	uc, err = proxy.ParseUpstreamsConfig(addrs, opts)
 	if err != nil {
