@@ -1,23 +1,15 @@
 package querylog
 
+import "github.com/AdguardTeam/AdGuardHome/internal/whois"
+
 // Client is the information required by the query log to match against clients
 // during searches.
 type Client struct {
-	WHOIS          *ClientWHOIS `json:"whois,omitempty"`
-	Name           string       `json:"name"`
-	DisallowedRule string       `json:"disallowed_rule"`
-	Disallowed     bool         `json:"disallowed"`
-	IgnoreQueryLog bool         `json:"-"`
-}
-
-// ClientWHOIS is the filtered WHOIS data for the client.
-//
-// TODO(a.garipov): Merge with home.RuntimeClientWHOISInfo after the
-// refactoring is done.
-type ClientWHOIS struct {
-	City    string `json:"city,omitempty"`
-	Country string `json:"country,omitempty"`
-	Orgname string `json:"orgname,omitempty"`
+	WHOIS          *whois.Info `json:"whois,omitempty"`
+	Name           string      `json:"name"`
+	DisallowedRule string      `json:"disallowed_rule"`
+	Disallowed     bool        `json:"disallowed"`
+	IgnoreQueryLog bool        `json:"-"`
 }
 
 // clientCacheKey is the key by which a cached client information is found.
