@@ -16,6 +16,7 @@ import (
 	"github.com/AdguardTeam/AdGuardHome/internal/aghalg"
 	"github.com/AdguardTeam/AdGuardHome/internal/aghhttp"
 	"github.com/AdguardTeam/AdGuardHome/internal/aghnet"
+	"github.com/AdguardTeam/golibs/errors"
 	"github.com/AdguardTeam/golibs/logutil/slogutil"
 	"github.com/AdguardTeam/golibs/timeutil"
 	"golang.org/x/net/idna"
@@ -219,10 +220,14 @@ func (l *queryLog) handleQueryLogConfig(w http.ResponseWriter, r *http.Request)
 func (l *queryLog) handlePutQueryLogConfig(w http.ResponseWriter, r *http.Request) {
 	ctx := r.Context()
 
-	newConf := &getConfigResp{}
-	err := json.NewDecoder(r.Body).Decode(newConf)
+	newConf, err := readConfigResp(r)
 	if err != nil {
-		aghhttp.ErrorAndLog(ctx, l.logger, r, w, http.StatusBadRequest, "%s", err)
+		code := http.StatusBadRequest
+		if errors.Is(err, ErrNullConfEnabled) || errors.Is(err, ErrNullAnonymizeIP) {
+			code = http.StatusUnprocessableEntity
+		}
+
+		aghhttp.ErrorAndLog(ctx, l.logger, r, w, code, "%s", err)
 
 		return
 	}
@@ -258,25 +263,48 @@ func (l *queryLog) handlePutQueryLogConfig(w http.ResponseWriter, r *http.Reques
 		return
 	}
 
-	if newConf.Enabled == aghalg.NBNull {
-		aghhttp.ErrorAndLog(ctx, l.logger, r, w, http.StatusUnprocessableEntity, "enabled is null")
+	l.applyQueryLogConfig(ctx, engine, ivl, newConf)
+}
 
-		return
+const (
+	// ErrNullConfEnabled is returned when [getConfigResp.Enabled] is not set.
+	ErrNullConfEnabled errors.Error = "enabled is null"
+
+	// ErrNullAnonymizeIP is returned when [getConfigResp.AnonymizeClientIP] is
+	// not set.
+	ErrNullAnonymizeIP errors.Error = "anonymize_client_ip is null"
+)
+
+// readConfigResp decodes and minimally validates the request body.  r must not
+// be nil.
+func readConfigResp(r *http.Request) (conf *getConfigResp, err error) {
+	conf = &getConfigResp{}
+	err = json.NewDecoder(r.Body).Decode(conf)
+	if err != nil {
+		// Don't wrap the error, because it's informative enough as is.
+		return nil, err
 	}
 
-	if newConf.AnonymizeClientIP == aghalg.NBNull {
-		aghhttp.ErrorAndLog(
-			ctx,
-			l.logger,
-			r,
-			w,
-			http.StatusUnprocessableEntity,
-			"anonymize_client_ip is null",
-		)
-
-		return
+	if conf.Enabled == aghalg.NBNull {
+		return nil, ErrNullConfEnabled
 	}
 
+	if conf.AnonymizeClientIP == aghalg.NBNull {
+		return nil, ErrNullAnonymizeIP
+	}
+
+	return conf, nil
+}
+
+// applyQueryLogConfig applies the validated config to queryLog.  engine must
+// not be nil.  ivl must pass [validateIvl], and newConf must be produced by
+// [readConfigResp].
+func (l *queryLog) applyQueryLogConfig(
+	ctx context.Context,
+	engine *aghnet.IgnoreEngine,
+	ivl time.Duration,
+	newConf *getConfigResp,
+) {
 	defer l.conf.ConfigModifier.Apply(ctx)
 
 	l.confMu.Lock()
