@@ -2,8 +2,7 @@
 
 verbose="${VERBOSE:-0}"
 
-if [ "$verbose" -gt '0' ]
-then
+if [ "$verbose" -gt '0' ]; then
 	set -x
 fi
 
@@ -14,29 +13,31 @@ set -e -f -u
 #
 # TODO(a.garipov): Add to helpers.sh and use more actively in scripts.
 log() {
-	if [ "$verbose" -gt '0' ]
-	then
+	if [ "$verbose" -gt '0' ]; then
 		# Don't use quotes to get word splitting.
 		echo "$1" 1>&2
 	fi
 }
 
-version="$( ./AdGuardHome_amd64 --version | cut -d ' ' -f 4 )"
-if [ "$version" = '' ]
-then
+# Allow developers to overwrite the command, e.g. for testing.
+snapcraft_cmd="${SNAPCRAFT_CMD:-snapcraft}"
+readonly snapcraft_cmd
+
+version="$(./AdGuardHome_amd64 --version | cut -d ' ' -f 4)"
+if [ "$version" = '' ]; then
 	log 'empty version from ./AdGuardHome_amd64'
+
 	exit 1
 fi
 readonly version
 
 log "version '$version'"
 
-for arch in\
-	'i386'\
-	'amd64'\
-	'armhf'\
-	'arm64'
-do
+for arch in \
+	'amd64' \
+	'arm64' \
+	'armhf' \
+	'i386'; do
 	build_output="./AdGuardHome_${arch}"
 	snap_output="./AdGuardHome_${arch}.snap"
 	snap_dir="${snap_output}.dir"
@@ -48,28 +49,13 @@ do
 	cp -r './snap/gui' "${snap_dir}/meta/"
 
 	# Create a snap.yaml file, setting the values.
-	sed\
-		-e 's/%VERSION%/'"$version"'/'\
-		-e 's/%ARCH%/'"$arch"'/'\
-		./snap/snap.tmpl.yaml\
-		> "${snap_dir}/meta/snap.yaml"
+	sed \
+		-e 's/%VERSION%/'"$version"'/' \
+		-e 's/%ARCH%/'"$arch"'/' \
+		./snap/snap.tmpl.yaml \
+		>"${snap_dir}/meta/snap.yaml"
 
-	# TODO(a.garipov): The snapcraft tool will *always* write everything,
-	# including errors, to stdout.  And there doesn't seem to be a way to change
-	# that.  So, save the combined output, but only show it when snapcraft
-	# actually fails.
-	set +e
-	snapcraft_output="$(
-		snapcraft pack "$snap_dir" --output "$snap_output" 2>&1
-	)"
-	snapcraft_exit_code="$?"
-	set -e
-
-	if [ "$snapcraft_exit_code" -ne '0' ]
-	then
-		log "$snapcraft_output"
-		exit "$snapcraft_exit_code"
-	fi
+	"$snapcraft_cmd" pack "$snap_dir" --output "$snap_output"
 
 	log "$snap_output"
 
